<?php
namespace Ep\Currency\Adapter;

/**
 * Dohvaća tečajnu listu sa https://openexchangerates.org/
 *
 * @package Ep\Currency\Adapter
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class OpenExchangeRates extends AbstractAdapter
{
    const BASE_URI = 'https://openexchangerates.org/api/';

    /**
     * Vraća tečajnu listu.
     *
     * @param string $baseCurrency
     *
     * @return array
     *
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function getRates($baseCurrency = self::BASE_CURRENCY): array
    {
        if ( !isset($this->options['app_id']) || empty($this->options['app_id'])) {
            throw new \RuntimeException("Parametar 'app_id' je obavezan.");
        }

        $response = $this->http->request('GET', 'latest.json', [
            'query' => [
                'app_id' => $this->options['app_id'],
            ],
        ]);

        $responseJson = $response->getBody()->getContents();
        $data = json_decode($responseJson, true);

        if (null === $data) {
            throw new \RuntimeException(sprintf(
                'Došlo je do greške prilikom dekodiranja JSONa: %s.',
                json_last_error_msg()
            ));
        }

        if ($baseCurrency === $data['base']) {
            $rates = $data['rates'];
        } else {
            $rates = $this->convertRates($baseCurrency, $data['rates']);
        }

        return [
            'timestamp' => $data['timestamp'],
            'base' => $baseCurrency,
            'rates' => $rates,
        ];
    }

    /**
     * Pretvara niz valuta iz jedne u drugu.
     *
     * @param string $to
     * @param array $rates
     *
     * @return array
     */
    private function convertRates(string $to, array $rates): array
    {
        if (!array_key_exists($to, $rates)) {
            throw new \RuntimeException(sprintf("Nepoznata valuta '%s'.", $to));
        }

        $target = $rates[$to];

        foreach ($rates as &$rate) {
            $ratio = $rate / $target;
            $rate = 1 / $ratio;
        }

        return $rates;
    }
}