<?php
namespace Ep\Middleware;

use Ep\Middleware\ErrorHandler\LoggerListener;
use Ep\Middleware\ErrorHandler\TemplatedResponse;
use Ep\Middleware\ErrorHandler\WhoopsResponse;
use Interop\Container\ContainerInterface;
use Zend\Diactoros\Response;
use Zend\ServiceManager\Factory\FactoryInterface;
use Zend\Stratigility\Middleware\ErrorHandler;
use Zend\View\Renderer\RendererInterface;

/**
 * Stvara novu instancu ErrorHandler middlewarea.
 *
 * @package Ep\Middleware
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
final class ErrorHandlerFactory implements FactoryInterface
{
    /**
     * Stvara middleware.
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return ErrorHandler
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $config = $container->get('config')['error_handler'] ?? [];
        $isDevelopment = $container->get('config')['app']['development'] ?? false;

        $responseGenerator = ($config['show_errors'] ?? false) || $isDevelopment
            ? new WhoopsResponse()
            : new TemplatedResponse($container->get(RendererInterface::class));

        $errorHandler = new ErrorHandler(function() {
            return new Response\HtmlResponse('500 Internal Server Error', 500);
        }, $responseGenerator);

        // Ako smo u devu, ne moramo logirati greške, vidimo ih.
        if (($config['log_errors'] ?? true) && !$isDevelopment) {
            /** @var \Zend\Log\Logger $logger */
            $logger = $container->get('app.logger');
            $errorHandler->attachListener(new LoggerListener($logger));
        }

        foreach ($config['listeners'] ?? [] as $listener) {
            if (!is_callable($listener)) {
                $listener = $container->get($listener);
            }
            $errorHandler->attachListener($listener);
        }

        return $errorHandler;
    }
}