<?php
namespace Ep\Middleware;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Zend\Expressive\Flash\FlashMessageMiddleware;
use Zend\Expressive\Session\SessionMiddleware;

/**
 * Omogućava korištenje objekata iz middlewareova u ostatku aplikacije.
 *
 * @package Ep\Middleware
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
final class MiddlewareBridge implements MiddlewareInterface
{
    const DEFAULT_ITEMS = [
        SessionMiddleware::SESSION_ATTRIBUTE,
        BaseUrl::BASE_URL,
        FlashMessageMiddleware::FLASH_ATTRIBUTE,
    ];

    /** @var array */
    private $items = [];

    /**
     * Vraća objekt s datim imenom.
     *
     * @param $name
     *
     * @return mixed
     */
    public function get(string $name)
    {
        if (!$this->has($name)) {
            throw new \RuntimeException(
                sprintf('Objekt s imenom `%s` ne postoji.', $name)
            );
        }

        return $this->items[$name];
    }

    /**
     * Da li objekt s datim imenom postoji?
     *
     * @param string $name
     *
     * @return bool
     */
    public function has(string $name): bool
    {
        return array_key_exists($name, $this->items);
    }

    /**
     * Postavlja objekt za ime.
     *
     * @param string $name
     * @param $item
     */
    public function set(string $name, $item): void
    {
        $this->items[$name] = $item;
    }

    /**
     * Process an incoming server request and return a response, optionally delegating
     * response creation to a handler.
     *
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     *
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        foreach (self::DEFAULT_ITEMS as $name) {
            if (($item = $request->getAttribute($name))) {
                $this->set($name, $item);
            }
        }

        return $handler->handle($request);
    }
}