<?php
namespace EpTest\Validator;

use Ep\Validator\MinVersion;
use EpTest\ContainerAwareTrait;
use PHPUnit\Framework\TestCase;
use Zend\ServiceManager\PluginManagerInterface;
use Zend\Validator\ValidatorPluginManager;

/**
 * Class MinVersionTest
 *
 * @package EpTest\Validator
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class MinVersionTest extends TestCase
{
    use ContainerAwareTrait;

    /**
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function testIsRegistered()
    {
        $container = $this->createContainer();
        /** @var PluginManagerInterface $validators */
        $validators = $container->get(ValidatorPluginManager::class);

        $this->assertTrue($validators->has('min_version'));
        $this->assertInstanceOf(MinVersion::class, $validators->get('min_version'));
    }

    public function testWillReturnFalseForEmptyValue()
    {
        $validator = new MinVersion();

        $this->assertFalse($validator->isValid(''));
        $this->assertArrayHasKey(MinVersion::EMPTY_VERSION, $validator->getMessages());
    }

    public function testWillReturnFalseForLowerVersion()
    {
        $validator = new MinVersion([
            'min_version' => '1.2.3',
        ]);

        $this->assertFalse($validator->isValid('1.2.2'));
        $this->assertArrayHasKey(MinVersion::INVALID_VERSION, $validator->getMessages());

        $message = $validator->getMessages()[MinVersion::INVALID_VERSION];

        $this->assertContains('1.2.3', $message);
        $this->assertContains('1.2.2', $message);
    }

    public function testWillReturnTrueForValidVersions()
    {
        $validator = new MinVersion([
            'min_version' => '1.2.3',
        ]);

        $this->assertTrue($validator->isValid('1.2.3'));
        $this->assertTrue($validator->isValid('2.0'));
    }
}