<?php
namespace EpTest\View\Helper;

use Ep\View\Helper\AssetUrl;
use EpTest\ContainerAwareTrait;
use PHPUnit\Framework\TestCase;
use Zend\ServiceManager\PluginManagerInterface;
use Zend\View\HelperPluginManager;
use Zend\View\Renderer\RendererInterface;

/**
 * Class AssetUrlTest
 *
 * @package EpTest\View\Helper
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class AssetUrlTest extends TestCase
{
    use ContainerAwareTrait;

    /**
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function testHelperHasTheProperBasePathHelper()
    {
        $container = $this->createContainer([
            'app' => [
                'asset_manifest' => __DIR__ . '/dist/manifest.json',
                'asset_url' => '',
            ],
        ]);
        /** @var PluginManagerInterface $helpers */
        $helpers = $container->get(HelperPluginManager::class);
        /** @var AssetUrl $assetUrl */
        $assetUrl = $helpers->get('asset_url');

        $this->assertEquals('/resolved.js', $assetUrl('asset.js'));

        $this->addToAssertionCount(1);
    }

    public function testRendererIsInjected()
    {
        $container = $this->createContainer([
            'app' => [
                'asset_manifest' => __DIR__ . '/dist/manifest.json',
                'asset_url' => '',
            ],
        ]);
        /** @var PluginManagerInterface $helpers */
        $helpers = $container->get(HelperPluginManager::class);
        /** @var AssetUrl $assetUrl */
        $assetUrl = $helpers->get('asset_url');

        $this->assertInstanceOf(RendererInterface::class, $assetUrl->getView());
    }
}